@@ -12,25 +12,29 @@ my $avatax
     = WebService::Avalara::AvaTax->new( map { ( $_ => $ENV{"AVALARA_\U$_"} ) }
         @AVALARA_ENV );
 
-my ( $answer_ref, $trace )
-    = $avatax->is_authorized( join q{, } => uniq map { $_->name }
+my $answer_ref
+    = $avatax->tax_svc_is_authorized( join q{, } => uniq map { $_->name }
         $avatax->wsdl->operations );
-
 if ( 'Success' eq $answer_ref->{parameters}{IsAuthorizedResult}{ResultCode} )
 {
-    say "$ENV{AVALARA_USERNAME} authorized for all SOAP operations";
+    say "$ENV{AVALARA_USERNAME} authorized for all tax services";
 }
+my @operations = split /\s*,\s*/ =>
+    $answer_ref->{parameters}{IsAuthorizedResult}{Operations};
+
+$answer_ref
+    = $avatax->address_svc_is_authorized( join q{, } => uniq map { $_->name }
+        $avatax->wsdl->operations );
+if ( 'Success' eq $answer_ref->{parameters}{IsAuthorizedResult}{ResultCode} )
+{
+    say "$ENV{AVALARA_USERNAME} authorized for all address services";
+}
+push @operations =>
+    split /\s*,\s*/ =>
+    $answer_ref->{parameters}{IsAuthorizedResult}{Operations};
 
 say
     "$ENV{AVALARA_USERNAME} expires $answer_ref->{parameters}{IsAuthorizedResult}{Expires}";
 
-say "Authorized methods:";
-for my $method (
-    uniq
-    split /\s*,\s*/ =>
-    $answer_ref->{parameters}{IsAuthorizedResult}{Operations}
-    )
-{
-    $method =~ s/ (?<= [[:alnum:]] ) ( [[:upper:]] ) /_\l$1/xmsg;
-    say lcfirst $method;
-}
+say "Authorized operations:";
+say for uniq @operations;
