@@ -11,5 +11,5 @@ my $avatax
     = WebService::Avalara::AvaTax->new( map { ( $_ => $ENV{"AVALARA_\U$_"} ) }
         @AVALARA_ENV );
 
-say $avatax->wsdl->explain( $_, PERL => 'INPUT', recurse => 1, %SOAP_PARAMS )
-    for map { $_->name } $avatax->wsdl->operations;
+say $_->explain( $avatax->wsdl, PERL => 'INPUT', recurse => 1 )
+    for $avatax->wsdl->operations;
